import { HttpMethod, Route, StatusCodes, Request, Response } from 'org.crazydoctor.expressts';
import XmlWriter from '../util/XmlWriter';
import fs from 'fs';
import * as path from 'path';
import { CommentsManager } from '../comments/CommentsManager';

class GetCommentsXml extends Route {

	private getCommentsXmlData(): string {
		const xmlWriter = new XmlWriter();
		const sourcesPath = CommentsManager.CommentsFSRoot;
		const files = fs.readdirSync(sourcesPath);

		const readRepoDir = (repoPath: string, className: string[]): void => {
			const files = fs.readdirSync(repoPath);
			let newClass = false;
			files.forEach(file => {
				const filePath = path.join(repoPath, file);
				if (fs.statSync(filePath).isDirectory()) {
					className.push(file);
					readRepoDir(filePath, className);
				} else {
					if(!newClass) {
						newClass = true;
						xmlWriter.startTag('class', { 'name': className.join('.') }); // <class>
					}
					const [editor, timestamp, ...rest] = fs.readFileSync(filePath, 'utf-8').split(':');
					const content = rest.join(':'); 
					xmlWriter.writeTag('comment', { 'property': file, 'editor': editor, 'timestamp': timestamp }, content);
				}
			});
			if (newClass) {
				className.pop();
				xmlWriter.endTag(); // </class>
			}
		};

		xmlWriter.startTag('root', { 'timestamp': new Date().getTime().toString() }); // <root>

		files.forEach(file => {
			const filePath = path.join(sourcesPath, file);
			if (fs.statSync(filePath).isDirectory() && file !== '.' && file !== '..' && file !== '.git') {
				const repo = file;
				
				xmlWriter.startTag('repo', { 'name': repo }); // <repo>
				readRepoDir(filePath, []);
				xmlWriter.endTag(); // </repo>
			}
		});

		xmlWriter.endTag(); // </root>

		return xmlWriter.toString();
	}

	protected action = (req: Request, res: Response): any => {
		let xmlData = '';

		try {
			xmlData = this.getCommentsXmlData();
		} catch(e) {
			this.context.logError(e);
			res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
			return;
		}
		
		res.set({
			'Content-Type': 'application/xml',
			'Content-Disposition': 'attachment; filename="comments.xml"'
		});
		res.status(StatusCodes.OK).send(xmlData);
	};

	protected method = HttpMethod.GET;
	protected order = 7;
	protected route = '/comments.xml';
}

export default GetCommentsXml;