import {HttpMethod, Route, StatusCodes, Request, Response} from 'org.crazydoctor.expressts';
import ServerApp from '..';
import { ISession } from '../session/ISession';
import DBEvents from '../db/DBEvents';

class GetDataStatisticsUser extends Route {
	protected action = (req: Request, res: Response): any => {
		const session = req.session as ISession;
		
		if(!session.isEditor) {
			res.status(StatusCodes.FORBIDDEN).send('Access denied');
			return;
		}

		if(ServerApp.SourcesUpdating) {
			res.status(StatusCodes.ACCEPTED).send('Sources are being updated');
			return;
		}

		const user = req.params.user;
		const { page, pageSize } = req.query;
		DBEvents.getUserEventsCount(user).then((count) => {
			DBEvents.getUserEvents(user, parseInt(page as string), parseInt(pageSize as string)).then((userEvents) => {
				res.set({
					'Content-Type': 'application/json'
				});
				res.status(StatusCodes.OK).send({ count: count, records: JSON.parse(JSON.stringify(userEvents).replaceAll('__static__', '')) });
			});
		});
	};

	protected method = HttpMethod.GET;
	protected order = 9;
	protected route = '/data/statistics/user/:user';
}

export default GetDataStatisticsUser;