import {HttpMethod, Route, StatusCodes, Request, Response} from 'org.crazydoctor.expressts';
import { PackageEntity, Sources } from '../sources/Sources';
import ServerApp from '..';
import { ISession } from '../session/ISession';
import { CommentsManager } from '../comments/CommentsManager';

class GetPackage extends Route {
	protected action = (req: Request, res: Response): any => {
		const session = req.session as ISession;
		
		if(ServerApp.SourcesUpdating) {
			res.status(StatusCodes.ACCEPTED).render('sources-update');
			return;
		}

		const packageName = req.params.packageName;
		const root = req.params.root;
		const pck: PackageEntity | null = Sources.findPackage(root, packageName);
		if(!pck) {
			res.status(StatusCodes.NOT_FOUND).render('class', { isAdmin: session.isAdmin || false, isEditor: session.isEditor || false, Login: session.login || null, Class: { name: packageName, isPackage: true }, ClassSource: '', template: 'class', title: 'Package not found', ClassList: Sources.getShortenedClassMap(), RepoNames: Sources.getRepoNames(), Z8Locales: null, Comments: null });
		} else {
			CommentsManager.getCommentsByClass(pck.root, pck.name).then((result) => {
				if(!result.success) {
					res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
					return;
				}

				const commentsObj: {[key: string] : any} = {};
				result.comments?.forEach((comment) => {
					commentsObj[comment.propertyName] = comment;
				});
				res.status(StatusCodes.OK).render('class', { Class: pck, ClassSource: null, isAdmin: session.isAdmin || false, isEditor: session.isEditor || false, Login: session.login || null, template: 'class', title: `Package: ${packageName}`, ClassList: Sources.getShortenedClassMap(), RepoNames: Sources.getRepoNames(), Z8Locales: Sources.Z8Locales, Comments: commentsObj });
			});
		}
	};
	protected method = HttpMethod.GET;
	protected order = 13;
	protected route = '/package/:root/:packageName';
}

export default GetPackage;