import {HttpMethod, Route, StatusCodes, Request, Response} from 'org.crazydoctor.expressts';
import { spawn } from 'child_process';
import * as path from 'path';
import ServerApp from '..';
import { Sources } from '../sources/Sources';
import { ISession } from '../session/ISession';

class PostUpdateSources extends Route {
	protected method = HttpMethod.POST;
	protected order = 1;
	protected route = '/updateSources';

	protected action = (req: Request, res: Response): any => {
		const session = req.session as ISession;
		
		if(ServerApp.SourcesUpdating) {
			res.status(StatusCodes.CONFLICT).send('The source update is already in progress.');
			return;
		}

		if(session.isAdmin)
			this.updateSources(res);
		else
			res.status(StatusCodes.FORBIDDEN).send('Forbidden.');
	};

	private updateSources(res: Response): void {
		try {
			this.context.logInfo('[POST /updateSources] Sources update started...');
			ServerApp.SourcesUpdating = true;
			const update = spawn('node', [path.resolve(__dirname, '../util/UpdateSources.js')]);
	
			update.stdout.on('data', (data) => {
				console.log(data.toString());
			});

			update.stderr.on('data', (data) => {
				console.error(data.toString());
			});

			update.on('close', (code) => {
				ServerApp.SourcesUpdating = false;
				if (code === 0) {
					Sources.get(() => {
						this.context.logInfo('[POST /updateSources] Sources update finished!');
						res.status(StatusCodes.OK).send('OK');
					});
				} else {
					this.context.logError('[POST /updateSources] Sources update failed.');
					res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
				}
			});
	
			update.on('error', (e) => {
				ServerApp.SourcesUpdating = false;
				this.context.logError('[POST /updateSources] Sources update failed.');
				res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
				console.error(e);
			});
		} catch (e) {
			ServerApp.SourcesUpdating = false;
			this.context.logError('[POST /updateSources] Sources update failed.');
			res.status(StatusCodes.INTERNAL_SERVER_ERROR).send('Internal server error');
			console.error(e);
		}
	}
}

export default PostUpdateSources;