import { CronJob } from 'cron';
import ServerApp from '..';
import { Sources } from '../sources/Sources';
import { spawn } from 'child_process';
import * as path from 'path';

class CronSourcesUpdateTask {
	private static sourcesUpdateTask(): void {
		if(ServerApp.SourcesUpdating)
			return;
		try {
			ServerApp.Server?.logInfo('[Cron] Sources update started...');
			ServerApp.SourcesUpdating = true;
			const update = spawn('node', [path.resolve(__dirname, './UpdateSources.js')]);
	
			update.stdout.on('data', (data) => {
				console.log(data.toString());
			});

			update.stderr.on('data', (data) => {
				console.error(data.toString());
			});

			update.on('close', (code) => {
				ServerApp.SourcesUpdating = false;
				if (code === 0) {
					Sources.get();
					ServerApp.Server?.logInfo('[Cron] Sources update finished.');
				} else {
					ServerApp.Server?.logError('[Cron] Sources update failed.');
				}
			});

			update.on('error', (e) => {
				ServerApp.SourcesUpdating = false;
				ServerApp.Server?.logError('[Cron] Sources update failed.');
				console.error(e);
			});
		} catch (e) {
			ServerApp.SourcesUpdating = false;
			ServerApp.Server?.logError('[Cron] Sources update failed.');
			console.error(e);
		}
	}

	public static start(): void {
		const timeZone = 'Europe/Moscow';
		const cronSchedule = '0 3 * * *';
		const job = new CronJob(cronSchedule, CronSourcesUpdateTask.sourcesUpdateTask, null, true, timeZone);
		job.start();
	}
};

export default CronSourcesUpdateTask;