
class Progress {

	private current: number;
	private total: number;
	private header: string;
	private finished: boolean;

	private constructor(header: string, total: number) {
		this.header = header;
		this.total = total;
		this.finished = false;
		this.current = 0;
	}

	public static start(header: string, total: number): Progress {
		process.stdout.write(header + '\n');
		return new Progress(header, total).next(0);
	}

	public sumTotal(sumTotal: number): Progress {
		this.total += sumTotal;
		return this;
	}

	public next(current?: number): Progress {
		if(current == null)
			current = this.current + 1;

		const percentage = Math.floor((current / this.total) * 100);
		const progressBarLength = 50;
		const progressBar = '█'.repeat(Math.floor((current / this.total) * progressBarLength));
		const spaces = '░'.repeat(progressBarLength - progressBar.length);

		process.stdout.write(`Progress: ${progressBar}${spaces} ${percentage}% ${current === this.total ? 'Finished!': `[${current}/${this.total}]` }\r`);

		this.current = current;
		if (current === this.total) {
			this.finish();
		}

		return this;
	}

	public finish(): void {
		if(!this.finished) {
			if(this.current !== this.total)
				this.next(this.total);
			process.stdout.write('\n');
			this.finished = true;
		}
	}
}

export default Progress;